@@ -93,28 +93,39 @@ Document::Document (Application *App, bool StandAlone, Window *window):
 	m_pCurOp = NULL;
 	m_bReadOnly = false;
 	SetActive ();
-	if (window && App)
-		App->AddWindow (window);
+	m_AllowClipboard = true;
 }
 
 Document::~Document ()
+{
+	Clear ();
+	if (m_pView)
+		delete m_pView;
+	pango_attr_list_unref (m_PangoAttrList);
+	if (m_Theme)
+		m_Theme->RemoveClient (this);
+	if (m_App)
+		static_cast<Application*> (m_App)->SetActiveDocument (NULL);
+}
+
+void Document::Clear ()
 {
 	m_bIsLoading = true;
 	if (m_pCurOp)
 		delete m_pCurOp;
 	m_pCurOp = NULL;
-	if (m_filename)
-		g_free (m_filename);
-	if (m_title)
-		g_free (m_title);
-	if (m_label)
-		g_free (m_label);
-	if (m_author)
-		g_free (m_author);
-	if (m_mail)
-		g_free (m_mail);
-	if (m_comment)
-		g_free (m_comment);
+	g_free (m_filename);
+	m_filename = NULL;
+	g_free (m_title);
+	m_title = NULL;
+	g_free (m_label);
+	m_label = NULL;
+	g_free (m_author);
+	m_author = NULL;
+	g_free (m_mail);
+	m_mail = NULL;
+	g_free (m_comment);
+	m_comment = NULL;
 	map<string, Object *>::iterator it;
 	Object *obj;
 	while (HasChildren ()) {
@@ -122,11 +133,14 @@ Document::~Document ()
 		obj->Lock ();
 		Remove (obj);
 	}
-	if (m_pView)
-		delete m_pView;
-	pango_attr_list_unref (m_PangoAttrList);
-	if (m_Theme)
-		m_Theme->RemoveClient (this);
+	while (!m_RedoList.empty ()) {
+		delete m_RedoList.front ();
+		m_RedoList.pop_front ();
+	}
+	while (!m_UndoList.empty ()) {
+		delete m_UndoList.front ();
+		m_UndoList.pop_front ();
+	}
 }
 
 GtkWidget* Document::GetWidget ()
@@ -925,41 +939,26 @@ void Document::OnProperties ()
 
 void Document::SetTitle (const gchar* title)
 {
-	if (m_title)
-		g_free (m_title);
-	if (*title)
-		m_title = g_strdup(title);
-	else m_title = NULL;
+	g_free (m_title);
+	m_title = (title && *title)? g_strdup (title): NULL;
 }
 
 void Document::SetAuthor (const gchar* author)
 {
-	if (m_author)
-		g_free (m_author);
-	if (*author)
-		m_author = g_strdup (author);
-	else
-		m_author = NULL;
+	g_free (m_author);
+	m_author = (author && *author)? g_strdup (author): NULL;
 }
 
 void Document::SetMail (const gchar* mail)
 {
-	if (m_mail)
-		g_free (m_mail);
-	if (*mail)
-		m_mail = g_strdup (mail);
-	else
-		m_mail = NULL;
+	g_free (m_mail);
+	m_mail = (mail && *mail)? g_strdup (mail): NULL;
 }
 
 void Document::SetComment (const gchar* comment)
 {
-	if (m_comment)
-		g_free (m_comment);
-	if (*comment)
-		m_comment = g_strdup (comment);
-	else
-		m_comment = NULL;
+	g_free (m_comment);
+	m_comment = (comment && *comment)? g_strdup (comment): NULL;
 }
 	
 void Document::AddObject (Object* pObject)
@@ -985,11 +984,14 @@ void Document::OnUndo ()
 		Op->Undo ();
 		m_UndoList.pop_front ();
 		m_RedoList.push_front (Op);
-		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", true);
+		if (m_Window)
+			m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", true);
+	}
+	if (m_Window) {
+		if (m_UndoList.empty ())
+			m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", false);
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	}
-	if (m_UndoList.empty ())
-		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", false);
-	m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	m_bUndoRedo = false;
 	Update ();
 	EmptyTranslationTable ();
@@ -1006,11 +1008,14 @@ void Document::OnRedo ()
 		Op->Redo ();
 		m_RedoList.pop_front ();
 		m_UndoList.push_front (Op);
-		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
+		if (m_Window)
+	m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
+	}
+	if (m_Window) {
+		if (m_RedoList.empty ())
+			m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	}
-	if (m_RedoList.empty ())
-		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
-	m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	m_bUndoRedo = false;
 	EmptyTranslationTable ();
 	SetDirty (m_LastStackSize != m_UndoList.size () || (m_LastStackSize > 0 && m_OpID != m_UndoList.front ()->GetID ()));
