@@ -25,6 +25,9 @@
 #include "config.h"
 #include "crystaldoc.h"
 #include "crystalview.h"
+#include "cylinder.h"
+#include "matrix.h"
+#include "sphere.h"
 #include "xml-utils.h"
 #include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
@@ -33,6 +36,11 @@
 #include <cmath>
 #include <cstring>
 #include <vector>
+#ifdef HAVE_OPENBABEL_2_2
+#	include <list>
+using namespace OpenBabel;
+#endif
+#include <GL/gl.h>
 
 #define __max(x,y)  ((x) > (y)) ? (x) : (y)
 #define __min(x,y)  ((x) < (y)) ? (x) : (y)
@@ -61,7 +69,12 @@ gchar const *LatticeName[] = {
 
 using namespace gcu;
 
+#ifdef HAVE_OPENBABEL_2_2
+CrystalDoc::CrystalDoc (Application *App): GLDocument (App),
+	m_SpaceGroup (NULL)
+#else
 CrystalDoc::CrystalDoc (Application *App): GLDocument (App)
+#endif
 {
 }
 
@@ -169,6 +182,33 @@ void CrystalDoc::ParseXMLTree (xmlNode* xml)
 			if (i < 14)
 				m_lattice = (gcLattices)i;
 			xmlFree (txt);
+#ifdef HAVE_OPENBABEL_2_2
+		} else if (!strcmp ((gchar*) node->name, "group")) {
+			SpaceGroup *group = new SpaceGroup ();
+			txt = (char*) xmlGetProp (node, (xmlChar*) "Hall");
+			if (txt) {
+				group->SetHallName (txt);
+				xmlFree (txt);
+			} else {
+				txt = (char*) xmlGetProp (node, (xmlChar*) "HM");
+				if (txt) {
+					group->SetHMName (txt);
+					xmlFree (txt);
+				}
+			}
+			xmlNodePtr child = node->children;
+			while (child) {
+				if (!strcmp ((char const*) child->name, "transform")) {
+					txt = (char*) xmlNodeGetContent (child);
+					if (txt) {
+						group->AddTransform (txt);
+						xmlFree (txt);
+					}
+				}
+				child = child->next;
+			}
+			m_SpaceGroup = group;
+#endif
 		} else if (!strcmp ((gchar*) node->name, "cell")) {
 			txt = (char*) xmlGetProp (node, (xmlChar*)"a");
 			if (txt) {
@@ -282,6 +322,24 @@ void CrystalDoc::Update()
 	//Establish list of atoms
 	CrystalAtomList::iterator i, iend = AtomDef.end ();
 	
+#ifdef HAVE_OPENBABEL_2_2
+	if (m_SpaceGroup) {
+		vector3 v;
+		list<vector3> d;
+		for (i = AtomDef.begin(); i != iend; i++) {
+			v.x () = (*i)->x ();
+			v.y () = (*i)->y ();
+			v.z () = (*i)->z ();
+			d = m_SpaceGroup->Transform (v);
+			list<vector3>::iterator vi, viend = d.end();
+			CrystalAtom atom (**i);
+			for (vi=d.begin (); vi!= viend; vi++) {
+				atom.SetCoords ((*vi).x(), (*vi).y(), (*vi).z());
+				Duplicate (atom);
+			}
+		}
+	} else
+#endif
 	for (i = AtomDef.begin(); i != iend; i++) {
 		Duplicate(**i);
 		switch (m_lattice) {
@@ -312,7 +370,7 @@ void CrystalDoc::Update()
 	}
 	
 	////////////////////////////////////////////////////////////
-	//Establish list of atoms
+	//Establish list of lines
 	CrystalLineList::iterator j, jend = LineDef.end();
 	for (j = LineDef.begin() ; j != jend ; j++)
 	{
@@ -492,56 +550,74 @@ void CrystalDoc::Update()
 	}
 }
 
-void CrystalDoc::Duplicate(CrystalAtom& Atom)
+void CrystalDoc::Duplicate (CrystalAtom& Atom)
 {
 	CrystalAtom AtomX, AtomY, AtomZ ;
 	AtomX = Atom ;
-	AtomX.Move(- floor(AtomX.x()-m_xmin), - floor(AtomX.y()-m_ymin), - floor(AtomX.z()-m_zmin)) ;
-	while (AtomX.x() <= m_xmax)
-	{
+	AtomX.Move (- floor (AtomX.x ()-m_xmin + 1e-7), - floor (AtomX.y ()-m_ymin + 1e-7), - floor (AtomX.z ()-m_zmin + 1e-7)) ;
+	while (AtomX.x () <= m_xmax + 1e-7) {
 		AtomY = AtomX ;
-		while (AtomY.y() <= m_ymax)
-		{
+		while (AtomY.y () <= m_ymax + 1e-7) {
 			AtomZ = AtomY ;
-			while (AtomZ.z() <= m_zmax)
-			{
-				Atoms.push_back(new CrystalAtom(AtomZ)) ;
-				AtomZ.Move(0,0,1) ;
+			while (AtomZ.z () <= m_zmax + 1e-7) {
+				Atoms.push_back (new CrystalAtom (AtomZ)) ;
+				AtomZ.Move (0,0,1) ;
 			}
-			AtomY.Move(0,1,0) ;
+			AtomY.Move (0,1,0) ;
 		}
-		AtomX.Move(1,0,0) ;
+		AtomX.Move (1,0,0) ;
 	}
 }
 
-void CrystalDoc::Duplicate(CrystalLine& Line)
+void CrystalDoc::Duplicate (CrystalLine& Line)
 {
 	CrystalLine LineX, LineY, LineZ ;
 	LineX = Line ;
-	LineX.Move(- floor(LineX.Xmin()-m_xmin), - floor(LineX.Ymin()-m_ymin), - floor(LineX.Zmin()-m_zmin)) ;
-	while (LineX.Xmax() <= m_xmax)
-	{
+	LineX.Move (- floor (LineX.Xmin ()-m_xmin + 1e-7), - floor (LineX.Ymin ()-m_ymin + 1e-7), - floor (LineX.Zmin ()-m_zmin + 1e-7)) ;
+	while (LineX.Xmax () <= m_xmax + 1e-7) {
 		LineY = LineX ;
-		while (LineY.Ymax() <= m_ymax)
-		{
+		while (LineY.Ymax () <= m_ymax + 1e-7) {
 			LineZ = LineY ;
-			while (LineZ.Zmax() <= m_zmax)
-			{
-				Lines.push_back(new CrystalLine(LineZ)) ;
-				LineZ.Move(0,0,1) ;
+			while (LineZ.Zmax () <= m_zmax + 1e-7) {
+				Lines.push_back (new CrystalLine (LineZ)) ;
+				LineZ.Move (0,0,1) ;
 			}
-			LineY.Move(0,1,0) ;
+			LineY.Move (0,1,0) ;
 		}
-		LineX.Move(1,0,0) ;
+		LineX.Move (1,0,0) ;
 	}
 }
-
-void CrystalDoc::Draw()
+	
+void CrystalDoc::Draw (Matrix &m)
 {
-	CrystalAtomList::iterator i;
-	for (i = Atoms.begin(); i != Atoms.end(); i++) (*i)->Draw();
-	CrystalLineList::iterator j;
-	for (j = Lines.begin(); j != Lines.end(); j++) (*j)->Draw();
+	vector3 v, v1;
+	Sphere sp (10);
+	glEnable (GL_RESCALE_NORMAL);
+	CrystalAtomList::iterator i, iend = Atoms.end ();
+	double red, green, blue, alpha;
+	for (i = Atoms.begin (); i != iend; i++) {
+		(*i)->GetCoords (&v.x (), &v.y (), &v.z ());
+		v = m * v;
+		(*i)->GetColor (&red, &green, &blue, &alpha);
+		glColor4d (red, green, blue, alpha) ;
+		sp.draw (v, (*i)->r ());
+	}
+	glEnable (GL_NORMALIZE);
+	CrystalLineList::iterator j, jend = Lines.end ();
+	Cylinder cyl (10);
+	for (j = Lines.begin (); j != jend; j++) {
+		v.x () = (*j)->X1 ();
+		v.y () = (*j)->Y1 ();
+		v.z () = (*j)->Z1 ();
+		v = m * v;
+		v1.x () = (*j)->X2 ();
+		v1.y () = (*j)->Y2 ();
+		v1.z () = (*j)->Z2 ();
+		v1 = m * v1;
+		(*j)->GetColor (&red, &green, &blue, &alpha);
+		glColor4d (red, green, blue, alpha) ;
+		cyl.draw (v, v1, (*j)->GetRadius ());
+	}
 }
 
 CrystalView* CrystalDoc::CreateNewView()
@@ -590,7 +666,34 @@ xmlDocPtr CrystalDoc::BuildXMLTree()
 		
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"lattice", (xmlChar*)LatticeName[m_lattice]);
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
-	
+#ifdef HAVE_OPENBABEL_2_2
+		if (m_SpaceGroup) {
+			node = xmlNewDocNode (xml, NULL, (xmlChar*) "group", NULL);
+			if (node)
+				xmlAddChild (xml->children, node);
+			else
+				throw (int) 0;
+			string name = m_SpaceGroup->GetHallName ();
+			if (name.length () != 0)
+				xmlNewProp (node, (xmlChar*) "Hall", (xmlChar*) name.c_str ());
+			else {
+				name = m_SpaceGroup->GetHMName ();
+				if (name.length () != 0)
+					xmlNewProp (node, (xmlChar*) "HM", (xmlChar*) name.c_str ());
+			}
+			xmlNodePtr child;
+			transform3dIterator i;
+			transform3d const *t = m_SpaceGroup->BeginTransform (i);
+			while (t) {
+				child = xmlNewDocNode (xml, NULL, (xmlChar*) "transform", (xmlChar const*) t->DescribeAsString ().c_str ());
+				if (child)
+					xmlAddChild (node, child);
+				else
+					throw (int) 0;
+				t = m_SpaceGroup->NextTransform (i);
+			}
+		}
+#endif
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"cell", NULL);
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		snprintf(buf, sizeof(buf), "%g", m_a);
@@ -654,3 +757,112 @@ xmlDocPtr CrystalDoc::BuildXMLTree()
 		return NULL;
 	}
 }
+
+#ifdef HAVE_OPENBABEL_2_2
+bool CrystalDoc::ImportOB (OBMol &mol)
+{
+	OBUnitCell *cell = dynamic_cast<OBUnitCell*> (mol.GetData (OBGenericDataType::UnitCell));
+	if (cell == NULL)
+		return false;
+	m_a = cell->GetA () * 100;
+	m_b = cell->GetB () * 100;
+	m_c = cell->GetC () * 100;
+	m_alpha = cell ->GetAlpha ();
+	m_beta = cell->GetBeta ();
+	m_gamma = cell->GetGamma ();
+	string const group_name = cell->GetSpaceGroupName ();
+	m_SpaceGroup = cell->GetSpaceGroup ();
+    if (!m_SpaceGroup)
+		return false;
+	int lattice = cell->GetLatticeType ();
+	switch (lattice) {
+	case OBUnitCell::Triclinic:
+		m_lattice = triclinic;
+		break;
+	case OBUnitCell::Monoclinic:
+		switch (group_name[0]) {
+		case 'C':
+			m_lattice = base_centered_monoclinic;
+			break;
+		default:
+			m_lattice = monoclinic;
+			break;
+		}
+		break;
+	case OBUnitCell::Orthorhombic:
+		switch (group_name[0]) {
+		case 'C':
+			m_lattice = base_centered_orthorhombic;
+			break;
+		case 'I':
+			m_lattice = body_centered_orthorhombic;
+			break;
+		case 'F':
+			m_lattice = face_centered_orthorhombic;
+			break;
+		default:
+			m_lattice = orthorhombic;
+			break;
+		}
+		break;
+	case OBUnitCell::Tetragonal:
+		switch (group_name[0]) {
+		case 'I':
+			m_lattice = body_centered_tetragonal;
+			break;
+		default:
+			m_lattice = tetragonal;
+			break;
+		}
+		break;
+	case OBUnitCell::Rhombohedral:
+		m_lattice = rhombohedral;
+		break;
+	case OBUnitCell::Hexagonal:
+		m_lattice = hexagonal;
+		break;
+	case OBUnitCell::Cubic:
+		switch (group_name[0]) {
+		case 'I':
+			m_lattice = body_centered_cubic;
+			break;
+		case 'F':
+			m_lattice = face_centered_cubic;
+			break;
+		default:
+			m_lattice = cubic;
+			break;
+		}
+		break;
+	}
+	matrix3x3 m = cell->GetFractionalMatrix ();
+	vector3 v;
+	// now get the atoms
+	OBAtomIterator it;
+	OBAtom *atom = mol.BeginAtom (it);
+	CrystalAtom *catom;
+	GcuAtomicRadius radius;
+	radius.type = GCU_VAN_DER_WAALS;
+	radius.charge = 0;
+    radius.cn = -1;
+    radius.spin = GCU_N_A_SPIN;
+    radius.scale = NULL;
+	while (atom) {
+		v.x () = atom->GetX();
+		v.y () = atom->GetY();
+		v.z () = atom->GetZ();
+		v *= m;
+		radius.Z = atom->GetAtomicNum ();
+		catom = new CrystalAtom (radius.Z, v.x (), v.y (), v.z ());
+		if (gcu_element_get_radius (&radius)) {
+			catom->SetRadius (radius);
+			catom->SetEffectiveRadiusRatio (.25);
+		}
+		AtomDef.push_back (catom);
+		atom = mol.NextAtom (it);
+	}
+
+	Update ();
+	return true;
+}
+#endif
