@@ -40,7 +40,7 @@
 
 static unsigned short nNewDocs = 1;
 
-gcApplication::gcApplication(): Application ("gcrystal")
+gcApplication::gcApplication(): Application ("gcrystal-unstable")
 {
 }
 
@@ -63,6 +63,9 @@ void gcApplication::OnFileOpen ()
 {
 	list<string> l;
 	l.push_front ("application/x-gcrystal");
+#ifdef HAVE_OPENBABEL_2_2
+	l.push_front ("chemical/x-cif");
+#endif
 	FileChooser (this, false, l);
 }
 
@@ -173,6 +176,7 @@ gcDocument* gcApplication::GetDoc (const char* filename)
 
 enum {
 	GCRYSTAL,
+	CIF,
 	VRML,
 	PIXBUF
 };
@@ -180,23 +184,28 @@ enum {
 bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc)
 {
 	gcDocument *Doc = static_cast<gcDocument*> (pDoc);
+	int type = GCRYSTAL;
 	if (!mime_type)
 		mime_type = "application/x-gcrystal";
 	string filename2 = filename;
 	if (bSave) {
-		int type = GCRYSTAL;
 		char const *pixbuf_type = NULL;
-		if (!strcmp (mime_type, "model/vrml"))
+		if (!strcmp (mime_type, "chemica/x-cif"))
+			type = CIF;
+		else if (!strcmp (mime_type, "model/vrml"))
 			type = VRML;
 		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
 			type = PIXBUF;
-		char *ext = NULL;
+		char const *ext = NULL;
 		switch (type) {
 		case GCRYSTAL:
-			ext = (char*) ".gcrystal";
+			ext = ".gcrystal";
+			break;
+		case CIF:
+			ext = ".cif";
 			break;
 		case VRML:
-			ext = (char*) ".wrl";
+			ext = ".wrl";
 			break;
 		default:
 			break;
@@ -229,14 +238,16 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 				GtkRecentData data;
 				data.display_name = (char*) Doc->GetTitle ();
 				data.description = NULL;
-				data.mime_type = (char*) "application/x-gcrystal";
-				data.app_name = (char*) "gcrystal";
-				data.app_exec = (char*) "gcrystal %u";
+				data.mime_type = const_cast<char*> ("application/x-gcrystal");
+				data.app_name = const_cast<char*> ("gcrystal");
+				data.app_exec = const_cast<char*> ("gcrystal %u");
 				data.groups = NULL;
 				data.is_private =  FALSE;
 				gtk_recent_manager_add_full (GetRecentManager (), filename2.c_str (), &data);
 				Doc->RenameViews ();
 				break;
+			case CIF:
+				break;
 			case VRML:
 				Doc->OnExportVRML (filename2);
 				break;
@@ -245,7 +256,10 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 				break;
 			}
 	} else {
-		if (strcmp (mime_type, "application/x-gcrystal"))
+		if (!strcmp (mime_type, "application/x-gcrystal"));
+		else if (!strcmp (mime_type, "chemical/x-cif"))
+			type = CIF;
+		else
 			return true;
 		gcDocument *xDoc = GetDoc (filename);
 		if (xDoc)
@@ -265,13 +279,18 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 					return true;
 			}
 		}
-		if (Doc->Load (filename)) {
+		if ((type == GCRYSTAL)? Doc->Load (filename):
+#ifdef HAVE_OPENBABEL_2_2
+						Doc->Import (filename, mime_type)) {
+#else
+						false) {
+#endif
 			GtkRecentData data;
 			data.display_name = (char*) Doc->GetTitle ();
 			data.description = NULL;
-			data.mime_type = (char*) "application/x-gcrystal";
-			data.app_name = (char*) "gcrystal";
-			data.app_exec = (char*) "gcrystal %u";
+			data.mime_type = const_cast<char*> (mime_type);
+			data.app_name = const_cast<char*> ("gcrystal");
+			data.app_exec = const_cast<char*> ("gcrystal %u");
 			data.groups = NULL;
 			data.is_private =  FALSE;
 			gtk_recent_manager_add_full (GetRecentManager (), filename, &data);
