@@ -25,11 +25,7 @@
 #include "config.h"
 #include "gcrystal.h"
 #include "atom.h"
-#ifdef GCU_OLD_VER
-#	include <chemistry/element.h>
-#else
-#	include <gcu/element.h>
-#endif
+#include <gcu/element.h>
 #include <glib.h>
 #include <GL/gl.h>
 #include <GL/glu.h>
