@@ -41,8 +41,8 @@
 #include <glade/glade.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
-//#include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <clocale>
 #include <cmath>
 #include <vector>
@@ -51,6 +51,11 @@
 #include <ostream>
 #include <sstream>
 #include <glib/gi18n.h>
+#ifdef HAVE_OPENBABEL_2_2
+#	include <openbabel/format.h>
+#	include <openbabel/obconversion.h>
+#	include <openbabel/math/matrix3x3.h>
+#endif
 
 #define SAVE	1
 #define LOAD	2
@@ -72,8 +77,8 @@ gcDocument::gcDocument (gcApplication *pApp) :CrystalDoc (pApp)
 
 gcDocument::~gcDocument()
 {
-	if (m_filename != NULL) g_free(m_filename);
-	if (m_title) g_free(m_title);
+	g_free(m_filename);
+	g_free(m_title);
 	Reinit();
 	Dialog *dialog;
 	while (!m_Dialogs.empty())
@@ -194,6 +199,12 @@ void gcDocument::SetTitle(const gchar* title)
 	m_title = g_strdup(title);
 }
 
+static int cb_xml_to_vfs (GnomeVFSHandle *handle, const char* buf, int nb)
+{
+	GnomeVFSFileSize ndone;
+	return (int) gnome_vfs_write (handle, buf, nb, &ndone);
+}
+
 void gcDocument::Save()
 {
 	if (!m_filename)
@@ -203,8 +214,31 @@ void gcDocument::Save()
 	try {
 		xml = BuildXMLTree();
 
-		if (xmlSaveFile (m_filename, xml) < 0)
-			Error (SAVE);
+			xmlIndentTreeOutput = true;
+			xmlKeepBlanksDefault (0);
+		
+			GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
+			gnome_vfs_get_file_info (m_filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
+
+			if (GNOME_VFS_FILE_INFO_LOCAL (info)) {
+				gnome_vfs_file_info_unref (info);
+				if (xmlSaveFormatFile (m_filename, xml, true) < 0) /*Error(SAVE)*/;
+			} else {
+				gnome_vfs_file_info_unref (info);
+				xmlOutputBufferPtr buf = xmlAllocOutputBuffer (NULL);
+				GnomeVFSHandle *handle;
+				GnomeVFSResult result = gnome_vfs_open (&handle, m_filename, GNOME_VFS_OPEN_WRITE);
+				if (result == GNOME_VFS_ERROR_NOT_FOUND)
+					result = gnome_vfs_create (&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0666);
+				if (result != GNOME_VFS_OK)
+					throw 1;
+				buf->context = handle;
+				buf->closecallback = (xmlOutputCloseCallback) gnome_vfs_close;
+				buf->writecallback = (xmlOutputWriteCallback) cb_xml_to_vfs;
+				int n = xmlSaveFormatFileTo (buf, xml, NULL, true);
+				if (n < 0)
+					throw 1;
+			}
 			
 		xmlFreeDoc (xml);
 		SetDirty (false);
@@ -250,13 +284,13 @@ bool gcDocument::Load (const string &filename)
 	oldtitle = g_strdup (m_title);
 	try {
 		if (SetFileName (filename), !m_filename || !m_title)
-			throw (int) 0;
-		if (!(xml = xmlParseFile (filename.c_str ())))
 			throw (int) 1;
-		if (xml->children == NULL)
+		if (!(xml = xmlParseFile (filename.c_str ())))
 			throw (int) 2;
-		if (strcmp ((char*) xml->children->name, "crystal"))
+		if (xml->children == NULL)
 			throw (int) 3;
+		if (strcmp ((char*) xml->children->name, "crystal"))
+			throw (int) 4;
 		if (oldfilename)
 			g_free(oldfilename);
 		g_free (oldtitle);
@@ -268,9 +302,10 @@ bool gcDocument::Load (const string &filename)
 		switch (num)
 		{
 		case 2:
+		case 3:
 			Error(XML);
 			break;
-		case 3:
+		case 4:
 			Error(FORMAT);
 			break;
 		default:
@@ -666,7 +701,100 @@ void gcDocument::RenameViews ()
 	}
 }
 
+#ifdef HAVE_OPENBABEL_2_2
 bool gcDocument::Import (const string &filename, const string& mime_type)
 {
+	gchar *oldfilename, *oldtitle;
+	if (m_filename)
+		oldfilename = g_strdup (m_filename);
+	else oldfilename = NULL;
+	oldtitle = g_strdup (m_title);
+	char *old_num_locale;
+	bool local;
+	GnomeVFSFileInfo *info = NULL;
+	bool result = false, read_only = false;
+	try {
+		if (!filename.length ())
+			throw (int) 0;
+		info = gnome_vfs_file_info_new ();
+		gnome_vfs_get_file_info (filename.c_str (), info, GNOME_VFS_FILE_INFO_DEFAULT);
+		local = GNOME_VFS_FILE_INFO_LOCAL (info);
+		if (!(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE)))
+			read_only = true;
+		gnome_vfs_file_info_unref (info);
+		if (SetFileName (filename), !m_filename || !m_title)
+			throw (int) 1;
+		if (oldfilename)
+			g_free(oldfilename);
+		g_free (oldtitle);
+		if (local) {
+			ifstream ifs;
+			GnomeVFSURI *uri = gnome_vfs_uri_new (filename.c_str ());
+			ifs.open (gnome_vfs_uri_get_path (uri));
+			gnome_vfs_uri_unref (uri);
+			if (ifs.fail ())
+				throw (int) 1;
+			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+			setlocale(LC_NUMERIC, "C");
+			OBMol Mol;
+			OBConversion Conv;
+			OBFormat* pInFormat = OBFormat::FormatFromMIME (mime_type.c_str ());
+			if (pInFormat == NULL)
+				throw 2;
+			Conv.SetInFormat (pInFormat);
+			Conv.Read (&Mol, &ifs);
+			result = ImportOB (Mol);
+			Mol.Clear ();
+			setlocale (LC_NUMERIC, old_num_locale);
+			g_free (old_num_locale);
+			ifs.close ();
+			if (!result)
+				throw (int) 3;
+		} else {
+			char *buf;
+			int size;
+			if (gnome_vfs_read_entire_file (filename.c_str (), &size, &buf) != GNOME_VFS_OK)
+				throw 1;
+			istringstream iss (buf);
+			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+			setlocale(LC_NUMERIC, "C");
+			OBMol Mol;
+			OBConversion Conv;
+			OBFormat* pInFormat = OBFormat::FormatFromMIME (mime_type.c_str ());
+			if (pInFormat == NULL)
+				throw 2;
+			Conv.SetInFormat (pInFormat);
+			Conv.Read (&Mol, &iss);
+			result = ImportOB (Mol);
+			Mol.Clear ();
+			setlocale (LC_NUMERIC, old_num_locale);
+			g_free (old_num_locale);
+			g_free (buf);
+			if (!result)
+				throw (int) 3;
+		}
+		UpdateAllViews ();
+		return true;
+	}
+	catch (int num) {
+		switch (num)
+		{
+		default:
+			Error(LOAD);
+		}
+		if (num >= 0) {
+			if (oldfilename)  {
+				SetFileName (oldfilename);
+				g_free (oldfilename);
+			} else {
+				g_free (m_filename);
+				m_filename = NULL;
+			}
+			SetTitle (oldtitle);
+			g_free (oldtitle);
+		}
+		return false;
+	}
 	return false;
 }
+#endif
