@@ -4,7 +4,7 @@
  * Gnome Crystal
  * main.cc 
  *
- * Copyright (C) 2000-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2000-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,7 +24,6 @@
 
 #include "config.h"
 #include <glade/glade.h>
-#include <stdio.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <gcu/element.h>
@@ -33,10 +32,12 @@
 #include "document.h"
 #include "view.h"
 #include "globals.h"
-#include <gtk/gtkglinit.h>
+#include <goffice/utils/go-file.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <gtk/gtkglinit.h>
 #include <glib/gi18n.h>
+#include <cstdio>
 
 using namespace gcu;
 using namespace std;
@@ -49,6 +50,28 @@ GtkWidget *mainwindow, *vbox1 ;
 GConfClient *conf_client;
 guint NotificationId;
 
+/* Following code is removed because libgnomeui is going to be deprecated
+* Just I don't know what will replace GnomeClient *
+static void session_die(GnomeClient *client, gpointer data)
+{
+   gtk_main_quit();
+}
+
+static gint session_save(GnomeClient *client, gint phase, GnomeSaveStyle save_style, gint is_shutdown, GnomeInteractStyle interact_style, gint is_fast, gpointer client_data)
+{
+	gchar **argv;
+	gint argc;
+	if (IsEmbedded())
+	{
+		argv = (gchar**) g_malloc0(sizeof(gchar*) * 3);
+		argc = 2;
+		argv[0] = (gchar*) client_data;
+		argv[1] = "--bonobo-server";
+		gnome_client_set_clone_command(client, argc, argv);
+		gnome_client_set_restart_command(client, argc, argv);
+	}
+}*/
+
 static void on_config_changed (GConfClient *client,  guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
 	g_return_if_fail (client == conf_client);
@@ -100,6 +123,7 @@ int main(int argc, char *argv[])
 	gtk_gl_init (&argc, &argv);
 	gnome_vfs_init ();
 	Element::LoadRadii ();
+//	gnome_authentication_manager_init ();
 	if (argc > 1 && argv[1][0] == '-') {
 		context = g_option_context_new (_(" [file...]"));
 		g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
@@ -152,7 +176,7 @@ int main(int argc, char *argv[])
 		path = gnome_vfs_uri_to_string (auri, GNOME_VFS_URI_HIDE_NONE);
 		if (bres)
 			pDoc = gcApp->OnFileNew ();
-		bres = gcApp->FileProcess (path, "application/x-gcrystal", false, NULL, pDoc);
+		bres = gcApp->FileProcess (path, go_get_mime_type (path), false, NULL, pDoc);
 		g_free (path);
 		gnome_vfs_uri_unref (auri);
 		argv++;
